@@ -173,6 +173,7 @@ int PartitionTable::numPrimaries() const
 void PartitionTable::append(Partition* partition)
 {
     children().append(partition);
+    std::sort(children().begin(), children().end(), [] (const Partition *a, const Partition *b) -> bool {return a->firstSector() < b->firstSector();});
 }
 
 /** @param f the flag to get the name for
@@ -215,8 +216,6 @@ QString PartitionTable::flagName(Flag f)
         return xi18nc("@item partition flag", "msft-data");
     case PartitionTable::FlagIrst:
         return xi18nc("@item partition flag", "irst");
-    case PartitionTable::FlagEsp:
-        return xi18nc("@item partition flag", "esp");
     default:
         break;
     }
@@ -246,7 +245,6 @@ const QList<PartitionTable::Flag> PartitionTable::flagList()
     rval.append(PartitionTable::FlagLegacyBoot);
     rval.append(PartitionTable::FlagMsftData);
     rval.append(PartitionTable::FlagIrst);
-    rval.append(PartitionTable::FlagEsp);
 
     return rval;
 }
@@ -273,7 +271,7 @@ QStringList PartitionTable::flagNames(Flags flags)
 
 bool PartitionTable::getUnallocatedRange(const Device& d, PartitionNode& parent, qint64& start, qint64& end)
 {
-    if (d.type() == Device::Disk_Device) {
+    if (d.type() == Device::Type::Disk_Device) {
         const DiskDevice& device = dynamic_cast<const DiskDevice&>(d);
         if (!parent.isRoot()) {
             Partition* extended = dynamic_cast<Partition*>(&parent);
@@ -294,7 +292,7 @@ bool PartitionTable::getUnallocatedRange(const Device& d, PartitionNode& parent,
         }
 
         return end - start + 1 >= PartitionAlignment::sectorAlignment(device);
-    } else if (d.type() == Device::LVM_Device) {
+    } else if (d.type() == Device::Type::LVM_Device || d.type() == Device::Type::SoftwareRAID_Device) {
         if (end - start + 1 > 0) {
             return true;
         }
@@ -318,13 +316,13 @@ Partition* createUnallocated(const Device& device, PartitionNode& parent, qint64
         r |= PartitionRole::Logical;
 
     // Mark unallocated space in LVM VG as LVM LV so that pasting can be easily disabled (it does not work yet)
-    if (device.type() == Device::LVM_Device)
+    if (device.type() == Device::Type::LVM_Device)
         r |= PartitionRole::Lvm_Lv;
 
     if (!PartitionTable::getUnallocatedRange(device, parent, start, end))
         return nullptr;
 
-    return new Partition(&parent, device, PartitionRole(r), FileSystemFactory::create(FileSystem::Unknown, start, end, device.logicalSize()), start, end, QString());
+    return new Partition(&parent, device, PartitionRole(r), FileSystemFactory::create(FileSystem::Type::Unknown, start, end, device.logicalSize()), start, end, QString());
 }
 
 /** Removes all unallocated children from a PartitionNode
@@ -378,7 +376,7 @@ void PartitionTable::insertUnallocated(const Device& d, PartitionNode* p, qint64
 
     qint64 lastEnd = start;
 
-    if (d.type() == Device::LVM_Device && !p->children().isEmpty()) {
+    if (d.type() == Device::Type::LVM_Device && !p->children().isEmpty()) {
         // rearranging the sectors of all partitions to keep unallocated space at the end
         lastEnd = 0;
         std::sort(children().begin(), children().end(), [](const Partition* p1, const Partition* p2) { return p1->deviceNode() < p2->deviceNode(); });
@@ -401,18 +399,26 @@ void PartitionTable::insertUnallocated(const Device& d, PartitionNode* p, qint64
         }
     }
 
-    // Take care of the free space between the end of the last child and the end
-    // of the device or the extended partition.
-    qint64 parentEnd = lastUsable();
-
-    if (!p->isRoot()) {
-        Partition* extended = dynamic_cast<Partition*>(p);
-        parentEnd = extended ? extended->lastSector() : -1;
-        Q_ASSERT(extended);
+    if (d.type() == Device::Type::LVM_Device)
+    {
+        const LvmDevice& lvm = static_cast<const LvmDevice&>(d);
+        p->insert(createUnallocated(d, *p, lastEnd, lastEnd + lvm.freePE() - 1));
     }
+    else
+    {
+        // Take care of the free space between the end of the last child and the end
+        // of the device or the extended partition.
+        qint64 parentEnd = lastUsable();
 
-    if (parentEnd >= firstUsable() && parentEnd >= lastEnd)
-        p->insert(createUnallocated(d, *p, lastEnd, parentEnd));
+        if (!p->isRoot()) {
+            Partition* extended = dynamic_cast<Partition*>(p);
+            parentEnd = extended ? extended->lastSector() : -1;
+            Q_ASSERT(extended);
+        }
+
+        if (parentEnd >= firstUsable() && parentEnd >= lastEnd)
+            p->insert(createUnallocated(d, *p, lastEnd, parentEnd));
+    }
 }
 
 /** Updates the unallocated Partitions for this PartitionTable.
@@ -427,7 +433,7 @@ void PartitionTable::updateUnallocated(const Device& d)
 qint64 PartitionTable::defaultFirstUsable(const Device& d, TableType t)
 {
     Q_UNUSED(t)
-    if (d.type() == Device::LVM_Device) {
+    if (d.type() == Device::Type::LVM_Device || d.type() == Device::Type::SoftwareRAID_Device) {
         return 0;
     }
 
@@ -455,6 +461,7 @@ static struct {
     { QLatin1String("dasd"), 1, false, true, PartitionTable::dasd },
     { QLatin1String("msdos"), 4, true, false, PartitionTable::msdos },
     { QLatin1String("msdos"), 4, true, false, PartitionTable::msdos_sectorbased },
+    { QLatin1String("dos"), 4, true, false, PartitionTable::msdos_sectorbased },
     { QLatin1String("dvh"), 16, true, true, PartitionTable::dvh },
     { QLatin1String("gpt"), 128, false, false, PartitionTable::gpt },
     { QLatin1String("loop"), 1, false, true, PartitionTable::loop },
@@ -516,7 +523,7 @@ bool PartitionTable::tableTypeIsReadOnly(TableType l)
 */
 bool PartitionTable::isSectorBased(const Device& d) const
 {
-    if (d.type() == Device::Disk_Device) {
+    if (d.type() == Device::Type::Disk_Device) {
         const DiskDevice& diskDevice = dynamic_cast<const DiskDevice&>(d);
 
         if (type() == PartitionTable::msdos) {
