@@ -1,5 +1,6 @@
 /*************************************************************************
  *  Copyright (C) 2016 by Chantara Tith <tith.chantara@gmail.com>        *
+ *  Copyright (C) 2018 by Andrius Štikonas <andrius@stikonas.eu>         *
  *                                                                       *
  *  This program is free software; you can redistribute it and/or        *
  *  modify it under the terms of the GNU General Public License as       *
@@ -15,18 +16,24 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.*
  *************************************************************************/
 
+#include "core/device_p.h"
 #include "core/volumemanagerdevice.h"
+#include "core/volumemanagerdevice_p.h"
 
 /** Constructs an abstract Volume Manager Device with an empty PartitionTable.
  *
+ * @param name the Device's name
+ * @param deviceNode the Device's node
+ * @param logicalExtentSize the logical extent size that device uses
 */
-VolumeManagerDevice::VolumeManagerDevice(const QString& name,
+VolumeManagerDevice::VolumeManagerDevice(std::shared_ptr<VolumeManagerDevicePrivate> d,
+                                         const QString& name,
                                          const QString& deviceNode,
-                                         const qint64 logicalSize,
+                                         const qint64 logicalExtentSize,
                                          const qint64 totalLogical,
                                          const QString& iconName,
                                          Device::Type type)
-    : Device(name, deviceNode, logicalSize, totalLogical, iconName, type)
+    : Device(std::static_pointer_cast<DevicePrivate>(d), name, deviceNode, logicalExtentSize, totalLogical, iconName, type)
 {
 }
 
@@ -34,3 +41,9 @@ QString VolumeManagerDevice::prettyDeviceNodeList() const
 {
     return deviceNodes().join(QStringLiteral(", "));
 }
+
+void VolumeManagerDevice::setTotalLogical(qint64 n)
+{
+    Q_ASSERT(n > 0);
+    d->m_TotalLogical = n;
+}
