@@ -25,7 +25,7 @@ FileSystem::CommandSupportType extended::m_Shrink = FileSystem::cmdSupportCore;
 FileSystem::CommandSupportType extended::m_Move = FileSystem::cmdSupportCore;
 
 extended::extended(qint64 firstsector, qint64 lastsector, qint64 sectorsused, const QString& label) :
-    FileSystem(firstsector, lastsector, sectorsused, label, FileSystem::Extended)
+    FileSystem(firstsector, lastsector, sectorsused, label, FileSystem::Type::Extended)
 {
 }
 
