@@ -1,7 +1,7 @@
 /*************************************************************************
  *  Copyright (C) 2012 by Volker Lanz <vl@fidra.de>                      *
  *  Copyright (C) 2015 by Teo Mrnjavac <teo@kde.org>                     *
- *  Copyright (C) 2016 by Andrius Štikonas <andrius@stikonas.eu>         *
+ *  Copyright (C) 2016-2018 by Andrius Štikonas <andrius@stikonas.eu>    *
  *                                                                       *
  *  This program is free software; you can redistribute it and/or        *
  *  modify it under the terms of the GNU General Public License as       *
@@ -29,62 +29,81 @@
 #include "util/capacity.h"
 #include "util/helpers.h"
 
-#include <blkid/blkid.h>
-
 #include <KLocalizedString>
 
+#include <QColor>
 #include <QFileInfo>
+#include <QStandardPaths>
 #include <QStorageInfo>
 
-const std::array< QColor, FileSystem::__lastType > FileSystem::defaultColorCode =
+const std::vector<QColor> FileSystem::defaultColorCode =
 {
 {
-    QColor( 220,205,175 ),
-    QColor( 187,249,207 ),
-    QColor( 102,121,150 ),
-    QColor( 122,145,180 ),
-    QColor( 143,170,210 ),
-    QColor( 155,155,130 ),
-    QColor( 204,179,215 ),
-    QColor( 229,201,240 ),
-    QColor( 244,214,255 ),
-    QColor( 216,220,135 ),
-    QColor( 251,255,157 ),
-    QColor( 200,255,254 ),
-    QColor( 137,200,198 ),
-    QColor( 210,136,142 ),
-    QColor( 240,165,171 ),
-    QColor( 151,220,134 ),
-    QColor( 220,205,175 ),
-    QColor( 173,205,255 ),
-    QColor( 176,155,185 ),
-    QColor( 170,30,77 ),
-    QColor( 96,140,85 ),
-    QColor( 33,137,108 ),
-    QColor( 250,230,255 ),
-    QColor( 242,155,104 ),
-    QColor( 160,210,180 ),
-    QColor( 255,170,0 ),
-    QColor( 170, 120, 255 ),
-    QColor( 177, 82, 69 )
+    QColor( 220,205,175 ), // unknown
+    QColor( 187,249,207 ), // extended
+    QColor( 102,121,150 ), // ext2
+    QColor( 122,145,180 ), // ext3
+    QColor( 143,170,210 ), // ext4
+    QColor( 155,155,130 ), // swap
+    QColor( 204,179,215 ), // fat16
+    QColor( 229,201,240 ), // fat32
+    QColor( 244,214,255 ), // ntfs
+    QColor( 216,220,135 ), // reiser
+    QColor( 251,255,157 ), // reiser4
+    QColor( 200,255,254 ), // xfs
+    QColor( 137,200,198 ), // jfs
+    QColor( 210,136,142 ), // hfs
+    QColor( 240,165,171 ), // hfs+
+    QColor( 151,220,134 ), // ufs
+    QColor( 220,205,175 ), // unformatted
+    QColor( 173,205,255 ), // btrfs
+    QColor( 176,155,185 ), // hpfs
+    QColor( 170,30,77 ),   // luks
+    QColor( 96,140,85 ),   // ocfs2
+    QColor( 33,137,108 ),  // zfs
+    QColor( 250,230,255 ), // exfat
+    QColor( 242,155,104 ), // nilfs2
+    QColor( 160,210,180 ), // lvm2 pv
+    QColor( 255,170,0 ),   // f2fs
+    QColor( 170,120,255 ), // udf
+    QColor( 177,82,69 ),   // iso9660
+    QColor( 223,39,104 ),  // luks2
+    QColor( 204,179,255 ), // fat12
+    QColor( 255,100,100 ), // linux_raid_member
+    QColor( 110,20,50 ),   // bitlocker
+    QColor( 255,155,174 ), // apfs
 }
 };
 
+struct FileSystemPrivate {
+    FileSystem::Type m_Type;
+    qint64 m_FirstSector;
+    qint64 m_LastSector;
+    qint64 m_SectorSize;
+    qint64 m_SectorsUsed;
+    QString m_Label;
+    QString m_UUID;
+};
 
 /** Creates a new FileSystem object
     @param firstsector the first sector used by this FileSystem on the Device
     @param lastsector the last sector used by this FileSystem on the Device
     @param sectorsused the number of sectors in use on the FileSystem
-    @param l the FileSystem label
-    @param t the FileSystem type
+    @param label the FileSystem label
+    @param type the FileSystem type
 */
-FileSystem::FileSystem(qint64 firstsector, qint64 lastsector, qint64 sectorsused, const QString& l, FileSystem::Type t) :
-    m_Type(t),
-    m_FirstSector(firstsector),
-    m_LastSector(lastsector),
-    m_SectorsUsed(sectorsused),
-    m_Label(l),
-    m_UUID()
+FileSystem::FileSystem(qint64 firstsector, qint64 lastsector, qint64 sectorsused, const QString& label, FileSystem::Type type) :
+    d(std::make_unique<FileSystemPrivate>())
+{
+    d->m_Type = type;
+    d->m_FirstSector = firstsector;
+    d->m_LastSector = lastsector;
+    d->m_SectorsUsed = sectorsused;
+    d->m_Label = label;
+    d->m_UUID = QString();
+}
+
+FileSystem::~FileSystem()
 {
 }
 
@@ -99,27 +118,6 @@ qint64 FileSystem::readUsedCapacity(const QString& deviceNode) const
     return -1;
 }
 
-static QString readBlkIdValue(const QString& deviceNode, const QString& tag)
-{
-    blkid_cache cache;
-    QString rval;
-
-    if (blkid_get_cache(&cache, nullptr) == 0) {
-        blkid_dev dev;
-
-        char* label = nullptr;
-        if ((dev = blkid_get_dev(cache, deviceNode.toLocal8Bit().constData(), BLKID_DEV_NORMAL)) != nullptr &&
-                (label = blkid_get_tag_value(cache, tag.toLocal8Bit().constData(), deviceNode.toLocal8Bit().constData()))) {
-            rval = QString::fromLocal8Bit(label);
-            free(label);
-        }
-
-        blkid_put_cache(cache);
-    }
-
-    return rval;
-}
-
 FileSystem::Type FileSystem::detectFileSystem(const QString& partitionPath)
 {
     return CoreBackendManager::self()->backend()->detectFileSystem(partitionPath);
@@ -127,7 +125,7 @@ FileSystem::Type FileSystem::detectFileSystem(const QString& partitionPath)
 
 QString FileSystem::detectMountPoint(FileSystem* fs, const QString& partitionPath)
 {
-    if (fs->type() == FileSystem::Lvm2_PV)
+    if (fs->type() == FileSystem::Type::Lvm2_PV)
         return FS::lvm2_pv::getVGName(partitionPath);
 
     if (partitionPath.isEmpty()) // Happens when during initial scan LUKS is closed
@@ -152,7 +150,7 @@ bool FileSystem::detectMountStatus(FileSystem* fs, const QString& partitionPath)
 {
     bool mounted = false;
 
-    if (fs->type() == FileSystem::Lvm2_PV) {
+    if (fs->type() == FileSystem::Type::Lvm2_PV) {
         mounted = FS::lvm2_pv::getVGName(partitionPath) != QString();
     } else {
         mounted = isMounted(partitionPath);
@@ -166,7 +164,7 @@ bool FileSystem::detectMountStatus(FileSystem* fs, const QString& partitionPath)
 */
 QString FileSystem::readLabel(const QString& deviceNode) const
 {
-    return readBlkIdValue(deviceNode, QStringLiteral("LABEL"));
+    return CoreBackendManager::self()->backend()->readLabel(deviceNode);
 }
 
 /** Creates a new FileSystem
@@ -361,7 +359,7 @@ bool FileSystem::updateUUID(Report& report, const QString& deviceNode) const
  */
 QString FileSystem::readUUID(const QString& deviceNode) const
 {
-    return readBlkIdValue(deviceNode, QStringLiteral("UUID"));
+    return CoreBackendManager::self()->backend()->readUUID(deviceNode);
 }
 
 /** Give implementations of FileSystem a chance to update the boot sector after the
@@ -381,13 +379,13 @@ bool FileSystem::updateBootSector(Report& report, const QString& deviceNode) con
 /** @return the minimum capacity valid for this FileSystem in bytes */
 qint64 FileSystem::minCapacity() const
 {
-    return 8 * Capacity::unitFactor(Capacity::Byte, Capacity::MiB);
+    return 8 * Capacity::unitFactor(Capacity::Unit::Byte, Capacity::Unit::MiB);
 }
 
 /** @return the maximum capacity valid for this FileSystem in bytes */
 qint64 FileSystem::maxCapacity() const
 {
-    return Capacity::unitFactor(Capacity::Byte, Capacity::EiB);
+    return Capacity::unitFactor(Capacity::Unit::Byte, Capacity::Unit::EiB);
 }
 
 /** @return the maximum label length valid for this FileSystem */
@@ -411,6 +409,11 @@ QString FileSystem::name(const QStringList& languages) const
     return nameForType(type(), languages);
 }
 
+FileSystem::Type FileSystem::type() const
+{
+    return d->m_Type;
+}
+
 /** @return a pointer to a QString C array with all FileSystem names */
 static const KLocalizedString* typeNames()
 {
@@ -444,6 +447,11 @@ static const KLocalizedString* typeNames()
         kxi18nc("@item filesystem name", "f2fs"),
         kxi18nc("@item filesystem name", "udf"),
         kxi18nc("@item filesystem name", "iso9660"),
+        kxi18nc("@item filesystem name", "luks2"),
+        kxi18nc("@item filesystem name", "fat12"),
+        kxi18nc("@item filesystem name", "linux_raid_member"),
+        kxi18nc("@item filesystem name", "BitLocker"),
+        kxi18nc("@item filesystem name", "apfs"),
     };
 
     return s;
@@ -454,10 +462,9 @@ static const KLocalizedString* typeNames()
 */
 QString FileSystem::nameForType(FileSystem::Type t, const QStringList& languages)
 {
-    Q_ASSERT(t >= 0);
-    Q_ASSERT(t < __lastType);
+    Q_ASSERT(t < Type::__lastType);
 
-    return typeNames()[t].toString(languages);
+    return typeNames()[static_cast<int>(t)].toString(languages);
 }
 
 /** @param s the name to get the type for
@@ -465,11 +472,11 @@ QString FileSystem::nameForType(FileSystem::Type t, const QStringList& languages
 */
 FileSystem::Type FileSystem::typeForName(const QString& s, const QStringList& languages )
 {
-    for (quint32 i = 0; i < __lastType; i++)
+    for (quint32 i = 0; i < static_cast<int>(Type::__lastType); i++)
         if (typeNames()[i].toString(languages) == s)
             return static_cast<FileSystem::Type>(i);
 
-    return Unknown;
+    return Type::Unknown;
 }
 
 /** @return a QList of all known types */
@@ -477,8 +484,8 @@ QList<FileSystem::Type> FileSystem::types()
 {
     QList<FileSystem::Type> result;
 
-    int i = Ext2; // first "real" filesystem
-    while (i != __lastType)
+    int i = static_cast<int>(Type::Ext2); // first "real" filesystem
+    while (i != static_cast<int>(Type::__lastType))
         result.append(static_cast<FileSystem::Type>(i++));
 
     return result;
@@ -548,9 +555,25 @@ bool FileSystem::unmount(Report& report, const QString& deviceNode)
     return false;
 }
 
+qint64 FileSystem::firstSector() const
+{
+    return d->m_FirstSector;
+}
+
+qint64 FileSystem::lastSector() const
+{
+        return d->m_LastSector;
+}
+
 bool FileSystem::findExternal(const QString& cmdName, const QStringList& args, int expectedCode)
 {
-    ExternalCommand cmd(cmdName, args);
+    QString cmdFullPath = QStandardPaths::findExecutable(cmdName);
+    if (cmdFullPath.isEmpty())
+        cmdFullPath = QStandardPaths::findExecutable(cmdName, { QStringLiteral("/sbin/"), QStringLiteral("/usr/sbin/"), QStringLiteral("/usr/local/sbin/") });
+    if (cmdFullPath.isEmpty())
+        return false;
+
+    ExternalCommand cmd(cmdFullPath, args);
     if (!cmd.run())
         return false;
 
@@ -566,3 +589,53 @@ FileSystem::SupportTool FileSystem::supportToolName() const
 {
     return SupportTool();
 }
+
+void FileSystem::setFirstSector(qint64 s)
+{
+    d->m_FirstSector = s;
+}
+
+void FileSystem::setLastSector(qint64 s)
+{
+    d->m_LastSector = s;
+}
+
+const QString& FileSystem::label() const
+{
+    return d->m_Label;
+}
+
+qint64 FileSystem::sectorSize() const
+{
+    return d->m_SectorSize;
+}
+
+qint64 FileSystem::sectorsUsed() const
+{
+    return d->m_SectorsUsed;
+}
+
+const QString& FileSystem::uuid() const
+{
+    return d->m_UUID;
+}
+
+void FileSystem::setSectorSize(qint64 s)
+{
+    d->m_SectorSize = s;
+}
+
+void FileSystem::setSectorsUsed(qint64 s)
+{
+    d->m_SectorsUsed = s;
+}
+
+void FileSystem::setLabel(const QString& s)
+{
+    d->m_Label = s;
+}
+
+void FileSystem::setUUID(const QString& s)
+{
+    d->m_UUID = s;
+}
