@@ -57,9 +57,9 @@ QList<Device*> DummyBackend::scanDevices(bool excludeLoop)
     return result;
 }
 
-Device* DummyBackend::scanDevice(const QString& device_node)
+Device* DummyBackend::scanDevice(const QString& deviceNode)
 {
-    DiskDevice* d = new DiskDevice(QStringLiteral("Dummy Device"), QStringLiteral("/tmp") + device_node, 255, 30, 63, 512);
+    DiskDevice* d = new DiskDevice(QStringLiteral("Dummy Device"), QStringLiteral("/tmp") + deviceNode, 255, 30, 63, 512);
     CoreBackend::setPartitionTableForDevice(*d, new PartitionTable(PartitionTable::msdos_sectorbased, 2048, d->totalSectors() - 2048));
     CoreBackend::setPartitionTableMaxPrimaries(*d->partitionTable(), 128);
     d->partitionTable()->updateUnallocated(*d);
@@ -74,36 +74,46 @@ FileSystem::Type DummyBackend::detectFileSystem(const QString& deviceNode)
 {
     Q_UNUSED(deviceNode)
 
-    return FileSystem::Unknown;
+    return FileSystem::Type::Unknown;
 }
 
-CoreBackendDevice* DummyBackend::openDevice(const QString& device_node)
+QString DummyBackend::readLabel(const QString& deviceNode) const
 {
-    DummyDevice* device = new DummyDevice(device_node);
+    Q_UNUSED(deviceNode)
 
-    if (device == nullptr || !device->open()) {
-        delete device;
+    return QString();
+}
+
+QString DummyBackend::readUUID(const QString& deviceNode) const
+{
+    Q_UNUSED(deviceNode)
+
+    return QString();
+}
+
+std::unique_ptr<CoreBackendDevice> DummyBackend::openDevice(const Device& d)
+{
+    std::unique_ptr<DummyDevice> device = std::make_unique<DummyDevice>(d.deviceNode());
+
+    if (!device->open())
         device = nullptr;
-    }
 
     return device;
 }
 
-CoreBackendDevice* DummyBackend::openDeviceExclusive(const QString& device_node)
+std::unique_ptr<CoreBackendDevice> DummyBackend::openDeviceExclusive(const Device& d)
 {
-    DummyDevice* device = new DummyDevice(device_node);
+    std::unique_ptr<DummyDevice> device = std::make_unique<DummyDevice>(d.deviceNode());
 
-    if (device == nullptr || !device->openExclusive()) {
-        delete device;
+    if (!device->openExclusive())
         device = nullptr;
-    }
 
     return device;
 }
 
-bool DummyBackend::closeDevice(CoreBackendDevice* core_device)
+bool DummyBackend::closeDevice(std::unique_ptr<CoreBackendDevice> coreDevice)
 {
-    return core_device->close();
+    return coreDevice->close();
 }
 
 #include "dummybackend.moc"
